package com.fuzamei.controller;

import com.fuzamei.dto.*;
import com.fuzamei.entity.Market;
import com.fuzamei.enums.CodeEnum;
import com.fuzamei.service.BillOrderActService;
import com.fuzamei.service.MarketService;
import com.fuzamei.util.PageData;
import com.fuzamei.util.ResponseUtil;
import com.fuzamei.vo.MarketVO;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * @file_name: MarketController
 * @Description：
 * @author: 刘林杰
 * @date: 五月 2018/5/4 10:40
 * @version：
 */
@Controller
@RequestMapping(value = "/market")
public class MarketController {

    private static Logger logger = Logger.getLogger(MarketController.class);

    @Autowired
    private MarketService marketService;

    @Autowired
    private BillOrderActService billOrderActService;

    /**
     * 对单据进行申请融资
     * @param fincanceInfoDto
     * @return
     */
    @PostMapping(value = "/financingBill")
    public ResponseUtil financingBill(@RequestBody FincanceInfoDto fincanceInfoDto){

        if(fincanceInfoDto.getFinancePayMoney() <= 0) {
            return ResponseUtil.getResult(CodeEnum.FINANCING_FALSE.getCode(), false, null);
        }
        if(fincanceInfoDto.getDiscountMoney() <= 0) {
            return ResponseUtil.getResult(CodeEnum.FINANCING_FALSE.getCode(), false, null);
        }

        marketService.financingBill(fincanceInfoDto);

        return ResponseUtil.getResult(CodeEnum.FINANCING_SUCCESS.getCode(), false, null);

    }
    
    /**
     * 申请质押
     * @param pledgeInfoDTO
     * @return
     */
    @RequestMapping(value = "/pledge", method = RequestMethod.POST)
    public @ResponseBody
    ResponseUtil applyPledge(@RequestBody PledgeInfoDTO pledgeInfoDTO) {
        int result = marketService.applyPledge(pledgeInfoDTO);
        if(result > 0) {
            return ResponseUtil.getResult(CodeEnum.COMMONS_SUCCESS.getCode(),true,result);
        }else {
            return ResponseUtil.getResult(CodeEnum.COMMONS_ERROR.getCode(), true, null);
        }
    }

    /**
     * 获取市场挂牌信息，由时间，出账企业等做筛选，默认为空，不筛选，返回第一页
     * @param baseDTO
     * @return
     */
    @RequestMapping(value="/selectByState", method = RequestMethod.POST)
    public @ResponseBody
    ResponseUtil selectByState(@RequestBody BaseDTO baseDTO) {
        PageData pageData = marketService.selectByState(baseDTO);
        if(pageData != null) {
            return ResponseUtil.getResult(CodeEnum.COMMONS_SUCCESS.getCode(),true,pageData);
        }else {
            return ResponseUtil.getResult(CodeEnum.COMMONS_ERROR.getCode(), true, null);
        }

    }

    /**
     * 对市场挂牌进行摘牌，查看融资信息
     * @param marketDTO 所摘牌的单据信息
     * @Return ResponseUtil 成功放回信息由枚举类定义，@See ResultMsgEnum
     */
    @RequestMapping(value = "/prePick", method = RequestMethod.POST)
    public @ResponseBody
    ResponseUtil preBuyBill(@RequestBody MarketDTO marketDTO) {
        Integer marketId = marketDTO.getId();
        MarketVO marketVO = marketService.selectById(marketId);
        if(marketVO != null) {
            return ResponseUtil.getResult(CodeEnum.COMMONS_SUCCESS.getCode(),true,marketVO);
        }else {
            return ResponseUtil.getResult(CodeEnum.COMMONS_ERROR.getCode(), true, null);
        }
    }

    /**
     * 确认对市场买断融资挂牌进行摘牌
     * @param pickDTO marketId 所摘牌的单据信息
     * @Return ResponseUtil 成功放回信息由枚举类定义，@See ResultMsgEnum
     */
    @RequestMapping(value = "/pickFinance", method = RequestMethod.POST)
    @Transactional
    public @ResponseBody
    ResponseUtil pickFinance(@RequestBody PickDTO pickDTO) {
        int result = marketService.pickFinance(pickDTO);
        if(result > 0) {
            return ResponseUtil.getResult(CodeEnum.COMMONS_SUCCESS.getCode(),true,result);
        }else {
            return ResponseUtil.getResult(CodeEnum.COMMONS_ERROR.getCode(), true, null);
        }
    }

    /**
     * 对市场挂牌进行摘牌，查看质押信息
     * @param marketDTO 所摘牌的单据信息
     * @Return ResponseUtil 成功放回信息由枚举类定义，@See ResultMsgEnum
     */
    @RequestMapping(value = "/prePickPledge", method = RequestMethod.POST)
    public @ResponseBody
    ResponseUtil prePickPledge(@RequestBody MarketDTO marketDTO) {
        Integer marketId = marketDTO.getId();
        MarketVO marketVO = marketService.selectById(marketId);
        if(marketVO != null) {
            return ResponseUtil.getResult(CodeEnum.COMMONS_SUCCESS.getCode(),true, marketVO);
        }else {
            return ResponseUtil.getResult(CodeEnum.COMMONS_ERROR.getCode(), true, null);
        }
    }

    /**
     * 确认对市场买断融资挂牌进行摘牌
     * @param pickDTO
     * @return
     */
    @RequestMapping(value = "pickPledge", method = RequestMethod.POST)
    public @ResponseBody
    ResponseUtil pickPledge(@RequestBody PickDTO pickDTO) {
        int result = marketService.pickPledge(pickDTO);
        if(result > 0) {
            return ResponseUtil.getResult(CodeEnum.COMMONS_SUCCESS.getCode(),true,result);
        }else {
            return ResponseUtil.getResult(CodeEnum.COMMONS_ERROR.getCode(), true, null);
        }
    }

    /**
     * 回购（未调）
     * @param buyBackInfoDTO
     * @return
     */
    @RequestMapping(value = "buyBack", method = RequestMethod.POST)
    public @ResponseBody
    ResponseUtil burBack(@RequestBody BuyBackInfoDTO buyBackInfoDTO) {
        int result = marketService.buyBack(buyBackInfoDTO);
        if(result > 0) {
            return ResponseUtil.getResult(CodeEnum.COMMONS_SUCCESS.getCode(),true,result);
        }else {
            return ResponseUtil.getResult(CodeEnum.COMMONS_ERROR.getCode(), true, null);
        }
    }

}
