package com.fuzamei.service.serviceimpl;

import com.fuzamei.common.constrain.StatusEnum;
import com.fuzamei.dao.AssetDao;
import com.fuzamei.dao.BillDao;
import com.fuzamei.dao.BillOrderActDao;
import com.fuzamei.dto.*;
import com.fuzamei.entity.Asset;
import com.fuzamei.entity.BillOrderAct;
import com.fuzamei.entity.Order;
import com.fuzamei.service.BillService;
import com.fuzamei.util.BeanCopyUtil;
import com.fuzamei.util.PageData;
import com.fuzamei.vo.BillDetailVo;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @file_name: BIllServiceImpl
 * @Description：
 * @author: 刘林杰
 * @date: 五月 2018/5/8 9:52
 * @version：
 */
@Service
public class BIllServiceImpl implements BillService {
    public static final Logger logger = Logger.getLogger(BillOrderServiceImpl.class);
    @Autowired
    private BillDao billDao;
    @Autowired
    private BillOrderActDao billOrderActDao;
    @Autowired
    private AssetDao assetDao;

    String getCurrentUserid(String userId){
        return userId;
    }

    String getCurrentEnterprise(String userId) {
        if("1".equals(userId)) return "出账企业";
        if("2".equals(userId)) return "收账企业";
        return "交易企业";
    }


    /**
     * 根据母表Id查询母表信息
     * @param id
     * @return
     */
    @Override
    public BillDetailVo getBillDetailInfo(String id) {
         return (BillDetailVo)BeanCopyUtil.beanCopy(billDao.selectBillDetail(id),BillDetailVo.class);
    }

    @Override
    public PageData getInitiateBillInfo(FilterDto filterDto) {
        Page page = null;
        try{
            page = PageHelper.startPage(filterDto.getPageIndex(),filterDto.getPageSize());
            //String token = filterDto.getToken();
            //Integer userId = Integer.parseInt(MD5UtilTest.convertMD5(token).substring(0,5));
            billDao.getInitiateBillInfo(filterDto);
        }catch (Exception e){
            e.printStackTrace();
        }
        return PageData.getPageData(page,InitiateBillVo.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertOrder(OrderDto orderDto) {
        //插入母表
        System.out.println(orderDto);
        Order order = (Order)BeanCopyUtil.beanCopy(orderDto,Order.class);
        order.setCreateTime(System.currentTimeMillis());
        System.out.println(order);
        billDao.insertOrder(order);

        //插入流转表
        BillOrderAct orderAct = (BillOrderAct) BeanCopyUtil.beanCopy(orderDto,BillOrderAct.class);
        orderAct.setMoney(orderDto.getSummary());
        orderAct.setOperatorEnterprise(orderDto.getOutEnterprise());
        orderAct.setPayEnterprise(orderDto.getOutEnterprise());
        orderAct.setCreateTime(System.currentTimeMillis());
        orderAct.setOwnCopies((int)orderDto.getSummary());
        orderAct.setCreateBy(orderDto.getOutEnterprise());
        orderAct.setOwnEnterprise(orderDto.getInEnterprise());
        orderAct.setStatus(StatusEnum.BILL_WAIT.getCode());
        billOrderActDao.insertIntoBillOrder(orderAct);
        return 0;
    }

    /**
     * 返回未处于冻结中的金额总额
     * @param orderId
     * @return
     */
    @Override
    public double getCashBillInfo(String orderId) {
        BillDetailVo billDetailVo = getBillDetailInfo(orderId);
        //查询所有处于冻结中的总额
        double frozenBillCash = billOrderActDao.selectBillFreeze(orderId);
        return billDetailVo.getSummary()-frozenBillCash;
    }

    /**
     * 订单兑付，目前对未差分的单据测试完成
     * @param billOrderActId
     * @param userId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int cashBill(List<Integer> billOrderActIdList,String userId) {

        //查询余额
        BillOrderAct act = billOrderActDao.selectBillOrderActById(billOrderActId);
        String orderId = act.getOrderId();
        Asset asset = assetDao.getAssetDetail(userId);
        double frozenMoney = billOrderActDao.selectBillFreeze(orderId);
        final double cashMoney  =  asset.getChinaYuan()-frozenMoney;

        if(cashMoney<0){
            return -1;
        }

        Order order = billDao.selectBillDetail(orderId);  //获取母表order的利率‘
        
        AssetChangeDto assetChangeDto = new AssetChangeDto();
        assetChangeDto.setAccountId(act.getPayEnterprise());
        assetChangeDto.setChinaYuan(asset.getChinaYuan());
        //修改流转表
        //for循环：根据orderId兑付所有该单据可兑付的子单据
        List<AssetChangeDto> list = new ArrayList<>();
        List<BillOrderAct> billOrderActs = billOrderActDao.selectOrderOfBill(act.getOrderId());
        for(BillOrderAct billOrderAct:billOrderActs){
            UpdateBillOrderActDto updateBillOrderActDto = new UpdateBillOrderActDto();
            updateBillOrderActDto.setId(billOrderAct.getId());
            updateBillOrderActDto.setStatus(StatusEnum.BILL_END.getCode());
            AssetChangeDto assetChangeDto1 = new AssetChangeDto();
            assetChangeDto1.setAccountId(billOrderAct.getOwnEnterprise());
            assetChangeDto1.setWaitPayOrderMoney(-billOrderAct.getMoney());
            assetChangeDto1.setChinaYuan(billOrderAct.getMoney());
            assetChangeDto.add(-billOrderAct.getMoney());
            list.add(assetChangeDto1);
            billOrderActDao.updateBillStatus(updateBillOrderActDto);
        }

        UpdateBillOrderActDto updateBillOrderActDto = new UpdateBillOrderActDto();
        updateBillOrderActDto.setId(act.getId());

        if(frozenMoney==0){
            int days = (int)(System.currentTimeMillis()-order.getDeadline())/86400;
            if(days>0){
                //计算逾期利息
                assetChangeDto.add(-order.getSummary()*order.getRate()*days/360);
            }
            updateBillOrderActDto.setStatus(StatusEnum.BILL_END.getCode());
            billOrderActDao.updateBillStatus(updateBillOrderActDto);

            list.add(assetChangeDto);
            assetDao.updateAssetWithList(list);
            return 2;
        }
        list.add(assetChangeDto);
        assetDao.updateAssetWithList(list);
        BillOrderAct  billOrderAct = (BillOrderAct)BeanCopyUtil.beanCopy(act,BillOrderAct.class);
        billOrderAct.setMoney(cashMoney);
        billOrderAct.setStatus(StatusEnum.BILL_END.getCode());
        billOrderActDao.insertIntoBillOrder(billOrderAct);
        updateBillOrderActDto.setMoney(frozenMoney);
        billOrderActDao.updateBillMoney(updateBillOrderActDto);
        return 1;
    }

    /**
     *
     * @param billId
     * @return
     */
    @Override
    public int cancelBill(String billId) {
        UpdateBillOrderActDto updateBillOrderActDto = new UpdateBillOrderActDto();
        updateBillOrderActDto.setStatus(StatusEnum.CANCEL.getCode());
        updateBillOrderActDto.setBillId(billId);
        logger.info(updateBillOrderActDto.toString());
        return billOrderActDao.updateBillStatusByOrderActId(updateBillOrderActDto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int confirmBill(String billId) {
//        //通过状态、流转表ID、持有企业查找流水表
//    	  int intBillId = Integer.parseInt(billId);
//        Condition condition = new Condition();
//        condition.setStatus(StatusEnum.BILL_WAIT.getCode());  //状态  1
//        condition.setBillId(intBillId);  
//        BillOrderAct billOrderAct = BillOrderActDao.selectBillOrderActByCondition(condition);
//        //更改收账企业资产表
//        Asset asset  = assetDao.getAssetDetail(getCurrentUserid("2"));
//        asset.setUseableOrderMoney(asset.getUseableOrderMoney()+billOrderAct.getMoney());
//        assetDao.updateAsset(asset);
        //更改出账企业资产表（待兑付）
        //更新流转表
        UpdateBillOrderActDto updateBillOrderActDto = new UpdateBillOrderActDto();
        updateBillOrderActDto.setStatus(StatusEnum.OWNING.getCode());
        updateBillOrderActDto.setBillId(billId);
        return billOrderActDao.updateBillStatusByOrderActId(updateBillOrderActDto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int editBill(OrderDto orderDto) {
        Order order = (Order)BeanCopyUtil.beanCopy(orderDto,Order.class);
        billDao.updateBill(order);
        //这里用了delete(没有传入流转表id)，是否合理
        billOrderActDao.deleteBillOrdersActByOrderId(orderDto.getOrderId());
        BillOrderAct orderAct = (BillOrderAct) BeanCopyUtil.beanCopy(orderDto,BillOrderAct.class);
        orderAct.setMoney(orderDto.getSummary());
        orderAct.setOperatorEnterprise(orderDto.getOutEnterprise());
        orderAct.setPayEnterprise(orderDto.getOutEnterprise());
        orderAct.setCreateTime(System.currentTimeMillis());
        orderAct.setOwnCopies((int)orderDto.getSummary());
        orderAct.setCreateBy(orderDto.getOutEnterprise());
        orderAct.setOwnEnterprise(orderDto.getInEnterprise());
        orderAct.setStatus(StatusEnum.BILL_WAIT.getCode());
        billOrderActDao.insertIntoBillOrder(orderAct);
        return 0;
    }

    @Override
    public int isBillIdUsed(int id) {
        return 0;
    }
}
