package com.fuzamei.service.serviceimpl;

import com.fuzamei.common.constrain.StatusEnum;
import com.fuzamei.common.exception.G33Exception;
import com.fuzamei.dao.AssetDao;
import com.fuzamei.dao.BillDao;
import com.fuzamei.dao.BillOrderActDao;
import com.fuzamei.dao.MarketDAO;
import com.fuzamei.dto.BillPayInfoDto;
import com.fuzamei.dto.Condition;
import com.fuzamei.dto.UpdateBillOrderActDto;
import com.fuzamei.entity.Asset;
import com.fuzamei.entity.BillOrderAct;
import com.fuzamei.entity.Market;
import com.fuzamei.enums.CodeEnum;
import com.fuzamei.service.BillOrderActService;
import com.fuzamei.util.BeanCopyUtil;
import com.fuzamei.util.HttpLog;
import com.fuzamei.vo.AssetDetailVo;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @file_name: BillOrderServiceImpl
 * @Description：
 * @author: 刘林杰
 * @date: 五月 2018/5/10 17:43
 * @version：
 */
@Service
public class BillOrderServiceImpl implements BillOrderActService {
    public static final Logger logger = Logger.getLogger(HttpLog.class);
    @Autowired
    private BillOrderActDao BillOrderActDao;
    @Autowired
    private BillDao billDao;
    @Autowired
    private AssetDao assetDao;
    @Autowired
    private MarketDAO marketDAO;

    String getCurrentUserid(String userId){
        return userId;
    }

    String getCurrentEnterprise(String userId) {
        if("1".equals(userId)) return "出账企业";
        if("2".equals(userId)) return "收账企业";
        return "交易企业";
    }

    @Override
    public int insertIntoOrderAct(BillOrderAct billOrderAct) {
        return BillOrderActDao.insertIntoBillOrder(billOrderAct);
    }
    @Override
    public int updateBillOrderActStatus(UpdateBillOrderActDto updateBillOrderActDto) {
        return BillOrderActDao.updateBillStatus(updateBillOrderActDto);
    }

    /**
     * 目前未对单据的状态做校验
     * @param billPayInfoDto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int payBill(BillPayInfoDto billPayInfoDto) {
        //这里的accountId应该由用户模块提供，为方便测试这样写
        //Asset asset = assetDao.getAssetDetail(billPayInfoDto.getAccountId());
        Asset asset = assetDao.getAssetDetail(getCurrentUserid("2"));
        //修改支付中单据金额
        asset.setPayingOrdermoney(asset.getPayingOrdermoney()+billPayInfoDto.getPayBillMoney());
        asset.setUseableOrderMoney(asset.getUseableOrderMoney()-billPayInfoDto.getPayBillMoney());
        assetDao.updateAsset(asset);

        //流转表记录操作
        BillOrderAct billOrderAct =  BillOrderActDao.selectBillOrderActById(billPayInfoDto.getBillId());
        double left = billOrderAct.getMoney() - billPayInfoDto.getPayBillMoney();
        logger.info("money left "+left);
        //这个判断位置？
        if(left>0){
            billOrderAct.setMoney(left);
            BillOrderActDao.updateBillOrderAct(billOrderAct);
            BillOrderAct billOrderActNew =(BillOrderAct) BeanCopyUtil.beanCopy(billOrderAct,BillOrderAct.class);
            billOrderActNew.setMoney(billPayInfoDto.getPayBillMoney());
            //billOrderActNew.setOperatorEnterprise(billPayInfoDto.getAccountId());
            billOrderActNew.setOperatorEnterprise(getCurrentEnterprise("2"));
            billOrderActNew.setOwnEnterprise(billPayInfoDto.getInEnterprise());
            billOrderActNew.setStatus(StatusEnum.PAY_WAIT.getCode());
            BillOrderActDao.insertIntoBillOrder(billOrderActNew);
        }
        if(left == 0){
            billOrderAct.setStatus(StatusEnum.PAY_WAIT.getCode());
            billOrderAct.setOwnEnterprise(billPayInfoDto.getInEnterprise());
            //billOrderAct.setOperatorEnterprise(billPayInfoDto.getAccountId());
            billOrderAct.setOperatorEnterprise(getCurrentEnterprise("2"));
            BillOrderActDao.updateBillOrderAct(billOrderAct);
        }
        if(left<0) {
            throw new G33Exception(CodeEnum.COMMONS_ERROR.getCode(),CodeEnum.COMMONS_ERROR.getMsg());
        }
        return 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int confirmPay(int billOrderActId, boolean confirm) {
        BillOrderAct billOrderAct = BillOrderActDao.selectBillOrderActById(billOrderActId);
        //这里的accountId应该由用户模块提供，为方便测试这样写，以下是更新相关企业资产
        //操作企业 这里是发出支付方
        //Asset asset = assetDao.getAssetDetail(billOrderAct.getOperatorEnterprise());
        Asset asset = assetDao.getAssetDetail(getCurrentUserid("2"));
        //修改支付中单据金额
        asset.setPayingOrdermoney(asset.getPayingOrdermoney()-billOrderAct.getMoney());

        //确认支付
        if(confirm){
            assetDao.updateAsset(asset);
            //持有企业 这里是收账企业
            asset = assetDao.getAssetDetail(billOrderAct.getOwnEnterprise());
            asset.setUseableOrderMoney(asset.getUseableOrderMoney()+billOrderAct.getMoney());
            assetDao.updateAsset(asset);

            //更改流转表数据
            Condition condition = (Condition)BeanCopyUtil.beanCopy(billOrderAct,Condition.class);
            logger.info(condition.toString());
            condition.setStatus(StatusEnum.OWNING.getCode());
            BillOrderAct billOrderAct1 = BillOrderActDao.selectBillOrderActByCondition(condition);
            if(billOrderAct1 != null){
                logger.info("billOrderAct1 is null");
                billOrderAct1.setMoney(billOrderAct1.getMoney()+billOrderAct.getMoney());
                BillOrderActDao.deleteBillOrdersActById(billOrderActId);
                return BillOrderActDao.updateBillOrderAct(billOrderAct1);
            }
            billOrderAct.setStatus(StatusEnum.OWNING.getCode());
            return BillOrderActDao.updateBillOrderAct(billOrderAct);
            }else {
            //拒绝支付
            asset.setUseableOrderMoney(asset.getUseableOrderMoney()+billOrderAct.getMoney());
            assetDao.updateAsset(asset);

           //更改流转表数据
            Condition condition = (Condition)BeanCopyUtil.beanCopy(billOrderAct,Condition.class);
            condition.setStatus(StatusEnum.OWNING.getCode());
            condition.setOwnEnterprise(billOrderAct.getOperatorEnterprise());
            logger.info(condition.toString());
            BillOrderAct billOrderAct1 = BillOrderActDao.selectBillOrderActByCondition(condition);
            if(billOrderAct1!=null){
                logger.info("billOrderAct1 != null");
                billOrderAct1.setMoney(billOrderAct1.getMoney()+billOrderAct.getMoney());
                billOrderAct1.setOwnEnterprise(billOrderAct.getOperatorEnterprise());
                BillOrderActDao.deleteBillOrdersActById(billOrderActId);
                return BillOrderActDao.updateBillOrderAct(billOrderAct1);
            }else{
                billOrderAct.setStatus(StatusEnum.OWNING.getCode());
                billOrderAct.setOwnEnterprise(billOrderAct.getOperatorEnterprise());
                return BillOrderActDao.updateBillOrderAct(billOrderAct);
            }
        }
    }

}
