package com.fuzamei.service.serviceimpl;


import com.fuzamei.common.constrain.StatusEnum;
import com.fuzamei.common.exception.G33Exception;
import com.fuzamei.dao.*;
import com.fuzamei.dto.*;
import com.fuzamei.entity.*;
import com.fuzamei.enums.CodeEnum;
import com.fuzamei.service.MarketService;
import com.fuzamei.util.BeanCopyUtil;
import com.fuzamei.util.PageData;
import com.fuzamei.vo.MarketVO;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.log4j.Logger;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
@Transactional
public class MarketServiceImpl implements MarketService {

    private Logger logger = Logger.getLogger(MarketServiceImpl.class);

    @Autowired
    private MarketDAO marketDAO;
    @Autowired
    private AssetDao assetDao;
    @Autowired
    private BillOrderActDao billOrderActDao;
    @Autowired
    private OrdersDAO ordersDAO;
    @Autowired
    private EnterpriseMapper enterpriseDao;


    /**
     * 显示市场行情
     * @param baseDTO
     * @return
     */
    @Override
    public PageData selectByState(BaseDTO baseDTO) {
        Page page = null;
        try {
            //传入页码以及每页的大小
            page = PageHelper.startPage(baseDTO.getPageIndex(), baseDTO.getPageSize());
            //startPage后面紧跟的这个查询就是一个分页查询
            marketDAO.selectByState(baseDTO);
        }catch (Exception e) {
            e.printStackTrace();
        }
        PageData pageData = PageData.getPageData(page, MarketVO.class);
        List<MarketVO> list = pageData.getList();
        for(MarketVO VO : list) {
            int remainingDays = (int) ((VO.getDeadline() - new Date().getTime())/1000/60/60/24);
            //int remainingDays = VO.getDeadline() /1000/60/60/24;
            VO.setRemainingDays(remainingDays);
        }

        return pageData;
    }

    /**
     * 通过市场id查找市场信息
     * @param marketId
     * @return
     */
    @Override
    public MarketVO selectById(Integer marketId) {
        Market Market = marketDAO.selectById(marketId);
        MarketVO marketVO = new MarketVO();
        BeanUtils.copyProperties(Market, marketVO);
        marketVO.setRemainingDays((int) ((marketVO.getDeadline() - new Date().getTime())/1000/60/60/24));
        return marketVO;
    }

    /**
     * 申请质押
     */
    @Override
    @Transactional
    public int applyPledge(PledgeInfoDTO pledgeInfoDTO) {

        //质押后更新流转表
        //通过流转表id获取流转表信息
        BillOrderAct billOrderAct = billOrderActDao.selectByOrderActId(pledgeInfoDTO.getOrderActId());
        //更新信息
        billOrderAct.setModifyTime(new Date().getTime());
        billOrderAct.setPledgeMoney(pledgeInfoDTO.getPledgeMoney());
        billOrderAct.setBuyBackDate(pledgeInfoDTO.getBuyBackDate());
        billOrderAct.setStatus(StatusEnum.PLEDGE_WAIT.getCode());
        billOrderActDao.updateOrderAct(billOrderAct);

        //添加新的市场表
        billOrderAct = billOrderActDao.selectByOrderActId(pledgeInfoDTO.getOrderActId());
        Order order = ordersDAO.selectByID(billOrderAct.getOrderId());
        Market market = Market.toMarket(order, billOrderAct);
        market.setOrderId(pledgeInfoDTO.getOrderActId());
        market.setTransferMoney(billOrderAct.getMoney());
        market.setAnnualRate(pledgeInfoDTO.getAnnualRate());
        market.setDiscountMoney(pledgeInfoDTO.getDiscountMoney());
        market.setStatus(1);
        market.setDelistStatus(1);
        market.setCreateTime(new Date().getTime());
        marketDAO.insertMarket(market);

        //更改资产表
        //获取资产表信息。待修改
        Asset asset = assetDao.getAssetDetail(pledgeInfoDTO.getAccountId());
        //更新资产表信息
        asset.setUseableOrderMoney(asset.getUseableOrderMoney() - pledgeInfoDTO.getPledgeMoney());
        asset.setPledgeOrderMoney(asset.getPledgeOrderMoney() + pledgeInfoDTO.getPledgeMoney());
        assetDao.updateAsset(asset);

        return 1;
    }


    /**
     * 摘取融资
     * @param pickDTO
     * @return
     */
    @Override
    @Transactional
    public int pickFinance(PickDTO pickDTO) {

        //获取市场信息
        Market market = marketDAO.selectById(pickDTO.getId());
        //获取操作企业资产信息，表完善后更改
        Asset asset = assetDao.getAssetDetail("1");
        //校验摘牌份额是否满足条件
        if(pickDTO.getPickCopies() > market.getTransferMoney()) {
            return -1;
        }
        if(asset.getChinaYuan() < pickDTO.getPickCopies()) {
            return -2;
        }
        //更新市场表信息
        market.setDiscountMoney(market.getDiscountMoney() - pickDTO.getDiscountMoney());
        market.setTransferMoney(market.getTransferMoney() - pickDTO.getPickCopies());
        market.setFinancingMoney(market.getFinancingMoney() - pickDTO.getPickCopies()
                + pickDTO.getDiscountMoney());
        marketDAO.updateMarket(market);
        //校验是否变为“已摘”
        marketDAO.updateDelistStatus(pickDTO);

        //添加流转表数据
        BillOrderAct ordersAct = new BillOrderAct();
        market = marketDAO.selectById(pickDTO.getId());
        BillOrderAct billOrderAct = billOrderActDao.selectByOrderActId(market.getOrderId());
        Order order = ordersDAO.selectByID(billOrderAct.getOrderId());
        //通过母表orderId、状态为持有中、持有企业来合并流转表
        ordersAct.setOrderId(billOrderAct.getOrderId());
        ordersAct.setOperatorEnterprise(pickDTO.getOperateEnterprise());
        ordersAct.setOperatorEnterpriseId(pickDTO.getOperateEnterpriseId());
        BillOrderAct oldBillOrderAct = billOrderActDao.selectSimilarOrderAct(ordersAct);
        if(oldBillOrderAct != null) {
            oldBillOrderAct.setMoney(oldBillOrderAct.getMoney() + pickDTO.getPickCopies());
            oldBillOrderAct.setModifyTime(new Date().getTime());
            return billOrderActDao.updateOrderAct(oldBillOrderAct);
        }else {
            ordersAct.setMoney(pickDTO.getPickCopies());
            ordersAct.setRate(market.getAnnualRate());
            ordersAct.setOwnEnterprise(pickDTO.getOperateEnterprise());
            ordersAct.setOwnEnterpriseId(pickDTO.getOperateEnterpriseId());
            ordersAct.setStatus(StatusEnum.OWNING.getCode());
            ordersAct.setCreateBy(pickDTO.getOperateEnterprise());
            ordersAct.setCreateTime(new Date().getTime());
            ordersAct.setModifyBy(pickDTO.getOperateEnterprise());
            ordersAct.setModifyTime(new Date().getTime());
            ordersAct.setPayEnterprise(order.getOutEnterprise());
            ordersAct.setPayEnterpriseId(order.getOutEnterpriseId());
            billOrderActDao.insertIntoBillOrder(ordersAct);
        }

        //更新流转表数据
        market = marketDAO.selectById(pickDTO.getId());
        billOrderAct = billOrderActDao.selectByOrderActId(market.getOrderId());
        billOrderAct.setMoney(billOrderAct.getMoney() - pickDTO.getPickCopies());
        billOrderAct.setModifyTime(new Date().getTime());
        billOrderActDao.updateOrderAct(billOrderAct);

        //买断摘牌后，更新持有单据企业资产表
        asset = assetDao.getAssetDetail("1");   //获取相应表字段后更改
        asset.setChinaYuan(asset.getChinaYuan() + pickDTO.getPickCopies() - pickDTO.getDiscountMoney());
        asset.setFinancingOrderMoney(asset.getFinancingOrderMoney() - pickDTO.getPickCopies());
        assetDao.updateAsset(asset);

        //买断摘牌后，更新操作企业资产表
        asset = assetDao.getAssetDetail("2");   //获取相应表字段后更改
        asset.setChinaYuan(asset.getChinaYuan() - pickDTO.getPickCopies() + pickDTO.getDiscountMoney());
        asset.setFinancingOrderMoney(asset.getFinancingOrderMoney() + pickDTO.getPickCopies());
        assetDao.updateAsset(asset);

        return 1;
    }

    /**
     * 摘取质押融资
     * @param pickDTO
     * @return
     */
    @Override
    @Transactional
    public int pickPledge(PickDTO pickDTO) {

        //通过市场id查询市场表数据
        Market Market = marketDAO.selectById(pickDTO.getId());
        //通过流转表id查询流转表数据
        BillOrderAct ownBillOrderAct = billOrderActDao.selectByOrderActId(Market.getOrderId());

        //更新市场表
        Market.setDelistStatus(0);
        marketDAO.updateMarket(Market);

        //通过操作企业和市场表数据添加新的流转表
        BillOrderAct newBillOrderAct = (BillOrderAct) BeanCopyUtil.beanCopy(ownBillOrderAct, BillOrderAct.class);
        newBillOrderAct.setOwnEnterprise(pickDTO.getOperateEnterprise());
        newBillOrderAct.setOwnEnterpriseId(pickDTO.getOperateEnterpriseId());
        newBillOrderAct.setStatus(StatusEnum.OWING_PLEDGE.getCode());
        newBillOrderAct.setModifyBy(pickDTO.getOperateEnterprise());
        newBillOrderAct.setModifyTime(new Date().getTime());
        billOrderActDao.insertIntoBillOrder(newBillOrderAct);

        //更新操作企业资产表
        Asset operatorAsset = assetDao.getAssetDetail("1"); //后续修改
        operatorAsset.setChinaYuan(operatorAsset.getChinaYuan() - pickDTO.getPickCopies());
        operatorAsset.setPickPledgeOrderMoney(operatorAsset.getPickPledgeOrderMoney() + pickDTO.getPickCopies());
        assetDao.updateAsset(operatorAsset);

        //更新原流转表
        ownBillOrderAct.setOperatorEnterpriseId(pickDTO.getOperateEnterpriseId());
        ownBillOrderAct.setOperatorEnterprise(pickDTO.getOperateEnterprise());
        ownBillOrderAct.setModifyBy(pickDTO.getOperateEnterprise());
        ownBillOrderAct.setModifyTime(new Date().getTime());
        ownBillOrderAct.setStatus(StatusEnum.PLEDGED.getCode());
        billOrderActDao.updateOrderAct(ownBillOrderAct);

        //更新持有企业资产表
        Asset ownEnterpriseAsset = assetDao.getAssetDetail("2");    //后续修改
        ownEnterpriseAsset.setChinaYuan(ownEnterpriseAsset.getChinaYuan() + pickDTO.getPickCopies());
        ownEnterpriseAsset.setPledgedToBeTakenOrderMoney(ownEnterpriseAsset.getPledgedToBeTakenOrderMoney() + pickDTO.getPickCopies());
        assetDao.updateAsset(ownEnterpriseAsset);

        return 1;
    }


    /**
     * 回购
     * @param buyBackInfoDTO
     * @return
     */
    @Override
    @Transactional
    public int buyBack(BuyBackInfoDTO buyBackInfoDTO) {
        //通过企业名称和流转表id查询质押方流转表
        BillOrderAct billOrderAct = new BillOrderAct();
        billOrderAct.setId(buyBackInfoDTO.getOrderActId());
        billOrderAct.setOwnEnterprise(buyBackInfoDTO.getOwnEnterprise());
        BillOrderAct pledgeOrderAct = billOrderActDao.selectPledgeOrderAct(billOrderAct);
        //通过流转表id和操作企业查询摘牌方流转表
        billOrderAct.setOwnEnterprise(pledgeOrderAct.getOperatorEnterprise());
        BillOrderAct pickOrderAct = billOrderActDao.selectPledgeOrderAct(billOrderAct);
        //更新质押方流转表
        pledgeOrderAct.setOperatorEnterprise(pledgeOrderAct.getOwnEnterprise());
        pledgeOrderAct.setOperatorEnterpriseId(pledgeOrderAct.getOwnEnterpriseId());
        pledgeOrderAct.setStatus(StatusEnum.OWNING.getCode());
        pledgeOrderAct.setModifyBy(buyBackInfoDTO.getOperatorEnterprise());
        pledgeOrderAct.setModifyTime(new Date().getTime());
        billOrderActDao.updateOrderAct(pledgeOrderAct);
        //更新摘牌方流转表
        pickOrderAct.setOperatorEnterprise(pledgeOrderAct.getOwnEnterprise());
        pickOrderAct.setOperatorEnterpriseId(pledgeOrderAct.getOwnEnterpriseId());
        pickOrderAct.setStatus(StatusEnum.BILL_END.getCode());
        pickOrderAct.setModifyBy(buyBackInfoDTO.getOperatorEnterprise());
        pickOrderAct.setModifyTime(new Date().getTime());
        billOrderActDao.updateOrderAct(pickOrderAct);
        //更新质押方资产表
        Asset pledgeAsset = assetDao.getAssetDetail("1");   //之后修改
        pledgeAsset.setChinaYuan(pledgeAsset.getChinaYuan() - buyBackInfoDTO.getPaidMoney());
        pledgeAsset.setPledgeOrderMoney(pledgeAsset.getPledgeOrderMoney() - billOrderAct.getFinancingMoney());
        pledgeAsset.setPledgedToBeTakenOrderMoney(pledgeAsset.getPledgedToBeTakenOrderMoney() - billOrderAct.getFinancingMoney());
        assetDao.updateAsset(pledgeAsset);
        //更新摘牌方流转表
        Asset pickAsset = assetDao.getAssetDetail("2"); //之后修改
        pickAsset.setChinaYuan(pickAsset.getChinaYuan() + buyBackInfoDTO.getPaidMoney());
        pickAsset.setPickPledgeOrderMoney(pickAsset.getPickPledgeOrderMoney() - billOrderAct.getFinancingMoney());
        assetDao.updateAsset(pickAsset);

        return 1;
    }

    /**
     * 单据融资
     * @param fincanceInfoDto
     * @return
     */
    @Override
    @Transactional
    public int financingBill(FincanceInfoDto fincanceInfoDto) {

        //获取账户id、企业名称和id
        String accountId = fincanceInfoDto.getAccountId();
        UserEnterprise userEnterprise = enterpriseDao.getUserEnterpriseByUserTel(accountId);

        //更新资产表
        Asset asset = assetDao.getAssetDetail(accountId);
        asset.setUseableOrderMoney(asset.getUseableOrderMoney() - fincanceInfoDto.getFinancePayMoney());
        asset.setFinancingOrderMoney(asset.getFinancingOrderMoney() + fincanceInfoDto.getFinancePayMoney());
        assetDao.updateAsset(asset);

        //判断流转表中剩余份额
        BillOrderAct billOrderAct = billOrderActDao.selectByOrderActId(fincanceInfoDto.getOrderActId());
        double leftMoney = billOrderAct.getMoney() - fincanceInfoDto.getFinancePayMoney();
        if(leftMoney > 0) {
            //更新旧流转表
            billOrderAct.setMoney(leftMoney);
            billOrderAct.setOperatorEnterprise(userEnterprise.getEnterpriseName());
            billOrderAct.setOperatorEnterpriseId(Integer.parseInt(userEnterprise.getEnterpriseAccount()));
            billOrderAct.setModifyBy(accountId);
            billOrderAct.setModifyTime(System.currentTimeMillis());
            billOrderActDao.updateBillOrderAct(billOrderAct);
            //增加新流转表
            BillOrderAct billOrderActNew = (BillOrderAct) BeanCopyUtil.beanCopy(billOrderAct, BillOrderAct.class);
            billOrderActNew.setMoney(fincanceInfoDto.getFinancePayMoney());
            billOrderActNew.setStatus(StatusEnum.FINANCE_WAITING.getCode());
            billOrderActNew.setCreateBy(accountId);
            billOrderActNew.setCreateTime(System.currentTimeMillis());
            billOrderActDao.insertIntoBillOrder(billOrderActNew);
        }else if(leftMoney == 0) {
            //更新流转表
            billOrderAct.setOperatorEnterprise(userEnterprise.getEnterpriseName());
            billOrderAct.setOperatorEnterpriseId(Integer.parseInt(userEnterprise.getEnterpriseAccount()));
            billOrderAct.setModifyBy(accountId);
            billOrderAct.setModifyTime(System.currentTimeMillis());
            billOrderAct.setStatus(StatusEnum.FINANCE_WAITING.getCode());
            billOrderActDao.updateBillOrderAct(billOrderAct);
        }else if(leftMoney < 0) {
            //异常
            throw new G33Exception(CodeEnum.COPIES_INSUFFICIENT.getCode(),CodeEnum.COPIES_INSUFFICIENT.getMsg());
        }

        //插入市场表
        Order order = ordersDAO.selectByID(billOrderAct.getOrderId());
        Market market = Market.toMarket(order, billOrderAct);
        market.setTransferMoney(fincanceInfoDto.getFinancePayMoney());
        market.setDiscountMoney(fincanceInfoDto.getDiscountMoney());
        market.setDiscountRate(fincanceInfoDto.getDiscountRate());
        market.setFinancingMoney(fincanceInfoDto.getFinanceGetMoney());
        market.setStatus(StatusEnum.MARKET_NOT_DELIST.getCode());
        marketDAO.insertMarket(market);

        return 1;
    }
}






















